#include <ESP8266WiFi.h>
#include <PubSubClient.h>
const char* WIFI_SSID = "<SSID>"; 
const char* WIFI_PASSWORD = "<Password>";
const char* WIFI_HOSTNAME = "<Hostname>";
const PROGMEM char* MQTT_ID = "CustomFirmwareClient";
const PROGMEM char* MQTT_IP = "<My-MQTT-Server-IP>";
const PROGMEM char* MQTT_USER = "<My-MQTT-Username>";
const PROGMEM char* MQTT_PASSWORD = "<My-MQTT-Password>";
const char* MQTT_STATE_TOPIC = "custom/test/status";
const char* MQTT_COMMAND_TOPIC = "custom/test/output";
const char* OUTPUT_ON = "ON";
const char* OUTPUT_OFF = "OFF";
const PROGMEM uint8_t OUTPUT_PIN = 5;
boolean output_state = false;
WiFiClient wifiClient;
PubSubClient client(wifiClient);

void publishOutputState() {
  if (output_state) { 
    client.publish(MQTT_STATE_TOPIC, OUTPUT_ON, true); 
  }
  else { 
    client.publish(MQTT_STATE_TOPIC, OUTPUT_OFF, true); 
  }
}
void setOutputState() {
  if (output_state) { 
    digitalWrite(OUTPUT_PIN, HIGH); 
  }
  else { 
    digitalWrite(OUTPUT_PIN, LOW); 
  }
}
void callback(char* p_topic, byte* p_payload, unsigned int p_length) {
  String payload;
  for (uint8_t i = 0; i < p_length; i++) { 
    payload.concat((char)p_payload[i]); 
  }
  if (String(MQTT_COMMAND_TOPIC).equals(p_topic)) {
    if (payload.equals(String(OUTPUT_ON))) {
      if (output_state != true) {
        output_state = true;
        setOutputState();
        publishOutputState();
      }
    } else if (payload.equals(String(OUTPUT_OFF))) {
      if (output_state != false) {
        output_state = false;
        setOutputState();
        publishOutputState();
      }
    }
  }
}
void reconnect() {
  while (!client.connected()) {
    if (client.connect(MQTT_ID, MQTT_USER, MQTT_PASSWORD)) {
      publishOutputState();
      client.subscribe(MQTT_COMMAND_TOPIC);
    } else { 
      delay(5000); 
    }
  }
}
void setup() {
  pinMode(OUTPUT_PIN, OUTPUT);
  setOutputState();
  WiFi.mode(WIFI_STA);
  WiFi.hostname(WIFI_HOSTNAME);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED) { 
    delay(500);
  }
  client.setServer(MQTT_IP, 1883);
  client.setCallback(callback);
}
void loop() {
  if (!client.connected()) { 
    reconnect(); 
  }
  client.loop();
}
